/*
 * BiffSocko
 * nostrobe.c
 *
 * nostrobe reports attempted connections to a logfile.  nostrobe 
 * comes set up to sit on port 1026 (PORT)(you can change this by changing
 * the define statement).  when a connection is made nostrobe reports 
 * to a logfile (LOGFILE).  This program is useful in detecting 
 * port scanners like strobe.  This program can also easily be modified
 * to send email notifications.  
 *
 * let me know if you find this program useful or in the case you have
 * any questions or comments about it, email me at:
 * biffsocko@mindspring.com
 *
 * COMPILE INSTRUCTIONS:
 * --------------------
 * nostrobe has been tested on IRIX 6.5, Linux, and Solaris 2.7
 * gcc -o nostrobe nostrobe.c
 * Solaris: gcc -o nostrobe nostrobe.c -lsocket -lnsl
 *
 * HOW TO RUN:
 * ----------
 * I suggest running nostrobe in your start up files
 * like /etc/rc2.d or /etc/rc.d/rc2.d but you can run it
 * at the command line if you want
 * 
 * EXIT CODES:
 * ----------
 * 0	Success
 * 1	Wrong Command Line Parameters
 * 2	Unable to open socket()
 * 3	Can't bind() to port 
 * 4	Unable to listen() to port
 * 5	Can't accept() connections
 * 6	Can't fork()
 * 7    error getting time
 * 8    error converting time to local time
 * 9    error opening logfile
 * 99	SIGINT
 * all other abnormal return exit codes from connection functions which do the 
 * recev/send stuff for this program
 *
 * Copyright (C) 1999 BiffSocko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */ 

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <time.h>

#define PORT 1026
#define BACKLOG 200
#define MAX 80
#define LOGFILE "/var/nostrobe"
/*****************************************************************************
 * 		Global Variables (yucky)
 *****************************************************************************/
int sd;                            /* socket discriptor for socket function */


/*****************************************************************************
 * 	Function Definitions 
 *****************************************************************************/
void closer(int);     /* this will close the open master socket upon SIGINT */

int main (int argc, char *argv[])
{
	int accept_sd;   			  /* socket discriptor for accept function */
	int alen;			   	  /* from adderlen */
	struct sockaddr_in home_addr;  	          /* network address structures */
	struct sockaddr_in connecting_addr;       /* network address structures */
	char LocalTime[MAX]; 			  /* just a place ta keep the time */
        struct tm *LOCtm;			  /* for local time */
        time_t clock;        		          /* holds num of seconds since jan 1 1970 */
	FILE *Fp=NULL;				  /* for logfile */

/*****************************************************************************
 * check for correct usage 
 *****************************************************************************/
if(argc != 1){
	fprintf(stderr, "%s does not accept arguements\n", argv[0]);
	exit(1);
}

/* shameless self promotion */
printf("\033[31m%s \033[36mversion 1\033[37m.\033[36m0 \033[31mBy BiffSocko\033[0m\n", argv[0]); 

/*****************************************************************************
 * open logfile
 *****************************************************************************/
if(!(Fp=fopen(LOGFILE, "w+"))){
	fprintf(stderr, "can't open %s for logs\n", LOGFILE);
	exit(9);
}

/*****************************************************************************
 * open socket
 *****************************************************************************/
if(!(sd=socket(AF_INET, SOCK_STREAM, 0))){
	fprintf(stderr, "%s is unable to create a socket. exiting\n", argv[0]);
	fclose(Fp);
	exit(2);
}

/*****************************************************************************
 * do the sockaddr_in thing for home_addr
 *****************************************************************************/
home_addr.sin_family = AF_INET;
home_addr.sin_port = htons(PORT);
home_addr.sin_addr.s_addr = INADDR_ANY;
bzero(&(home_addr.sin_zero), 0);


/*****************************************************************************
 * binding to port
 *****************************************************************************/
if((bind(sd, (struct sockaddr *)&home_addr, sizeof(struct sockaddr))) == -1){
	fprintf(stderr, "%s could not bind to port. exiting\n", argv[0]);
	close(sd);
	fclose(Fp);
	exit(3);
}

/*****************************************************************************
 * listen to the port
 *****************************************************************************/
if((listen(sd, BACKLOG)) == -1){
	fprintf(stderr, "%s unable to listen to port. exiting\n", argv[0]);
	close(sd);
	fclose(Fp);
	exit(4);
}


/*****************************************************************************
 * clean up runaway processes & catch signals for closing out nicely
 *****************************************************************************/
 signal(SIGCHLD, SIG_IGN);
 signal(SIGINT, closer);

/*****************************************************************************
 * this is the part that does the work, we will accept() the connection, 
 * and then fork() off and run the *connection functions*  
 *****************************************************************************/
while(1){
	alen=sizeof(connecting_addr);
	if(!(accept_sd=accept(sd, (struct sockaddr *) &connecting_addr, &alen))){
		fprintf(stderr, "%s is unable to accept connections. exiting\n", argv[0]);
		close(sd);
		fclose(Fp);
		exit(5);
	}

	/* fork off */
	switch(fork()){
	case 0:    /* child process */
		   (void) close(sd);

		   /*****************************************************************************
 		    * get time
 		    *****************************************************************************/
		   if((time(&clock)) == -1){
			fprintf(stderr, "error getting time()\n");
                	exit( 7);
        	   }
	           LOCtm=localtime(&clock);

		   if((strftime(LocalTime, MAX,"%d-%b-%y %H:%M", LOCtm))==0){
			fprintf(stderr, "error getting Local Time\n");
                	exit( 8);
        	   }

		   /* send infoz to logfile */
		   fprintf(Fp, "IP ADDRESS:%s  -  TIME:%s\n", inet_ntoa(connecting_addr.sin_addr),LocalTime);

		   exit(0);
	default:  /* parent process */
			close(accept_sd);
			break;
	case -1:  
			fprintf(stderr, "%s error forking child\n", argv[0]);
	}

}
close(sd);
fclose(Fp);
exit(0);
}

/*****************************************************************************
 * 	closer()
 * this will take care of closing up the master socket descriptor upon 
 * SIGINT or other signals.  Eventually, this will also log termination 
 * errors to syslog
 *****************************************************************************/
void closer(int SIGX)
{

	printf("exiting on Signal %d\n", SIGX);
	close(sd);
	exit(99);
}
